#include "config.h"

#include <epan/packet.h>

#define OPENHPSDR_PORT 1024

static int proto_openhpsdr = -1;
static gint ett_openhpsdr = -1;
static gint ett_openhpsdr_flags = -1;
static gint ett_openhpsdr_usbpacket = -1;
static gint ett_openhpsdr_c0tree = -1;
static gint ett_openhpsdr_c0 = -1;
static gint ett_openhpsdr_c1 = -1;
static gint ett_openhpsdr_c2 = -1;
static gint ett_openhpsdr_c3 = -1;
static gint ett_openhpsdr_c4 = -1;

static int hf_openhpsdr_command = -1;
static int hf_openhpsdr_metis_mac = -1;
static int hf_openhpsdr_metis_version = -1;
static int hf_openhpsdr_board_id = -1;
static int hf_openhpsdr_ep4_state = -1;
static int hf_openhpsdr_ep6_state = -1;
static int hf_openhpsdr_endpoint = -1;
static int hf_openhpsdr_sequence = -1;
static int hf_openhpsdr_usbpacket_c0 = -1;
static int hf_openhpsdr_usbpacket_ep6_rr = -1;
static int hf_openhpsdr_usbpacket_ep2_rr = -1;
static int hf_openhpsdr_ptt = -1;
static int hf_openhpsdr_dash = -1;
static int hf_openhpsdr_dot = -1;
static int hf_openhpsdr_overflow = -1;
static int hf_openhpsdr_i01 = -1;
static int hf_openhpsdr_i02 = -1;
static int hf_openhpsdr_i03 = -1;
static int hf_openhpsdr_i04 = -1;
static int hf_openhpsdr_cyclops_lock = -1;
static int hf_openhpsdr_cyclops_frequency_changed = -1;
static int hf_openhpsdr_usbpacket_mercury_version = -1;
static int hf_openhpsdr_usbpacket_penelope_version = -1;
static int hf_openhpsdr_usbpacket_metis_version = -1;
static int hf_openhpsdr_hermes_forward_power = -1;
static int hf_openhpsdr_apollo_forward_power = -1;
static int hf_openhpsdr_apollo_reverse_power = -1;
static int hf_openhpsdr_ain3 = -1;
static int hf_openhpsdr_ain4 = -1;
static int hf_openhpsdr_ain6 = -1;
static int hf_openhpsdr_merc1_version = -1;
static int hf_openhpsdr_merc1_overflow = -1;
static int hf_openhpsdr_merc2_version = -1;
static int hf_openhpsdr_merc2_overflow = -1;
static int hf_openhpsdr_merc3_version = -1;
static int hf_openhpsdr_merc3_overflow = -1;
static int hf_openhpsdr_merc4_version = -1;
static int hf_openhpsdr_merc4_overflow = -1;
static int hf_openhpsdr_mox = -1;
static int hf_openhpsdr_sample_data = -1;

/* C1 Fields from computer when C0 = 0x00 */
static int hf_openhpsdr_ep2_c1 = -1;
static int hf_openhpsdr_speed = -1;
static int hf_openhpsdr_10mhz = -1;
static int hf_openhpsdr_122mhz = -1;
static int hf_openhpsdr_config = -1;
static int hf_openhpsdr_mic_source = -1;

static const int *openhpsdr_ep2_c1_fields[] = {
	&hf_openhpsdr_speed,
	&hf_openhpsdr_10mhz,
	&hf_openhpsdr_122mhz,
	&hf_openhpsdr_config,
	&hf_openhpsdr_mic_source,
	NULL
};

/*  C2 Fields from computer when C0 = 0x00 */
static int hf_openhpsdr_ep2_c2 = -1;
static int hf_openhpsdr_mode = -1;
static int hf_openhpsdr_oc0 = -1;
static int hf_openhpsdr_oc1 = -1;
static int hf_openhpsdr_oc2 = -1;
static int hf_openhpsdr_oc3 = -1;
static int hf_openhpsdr_oc4 = -1;
static int hf_openhpsdr_oc5 = -1;
static int hf_openhpsdr_oc6 = -1;

static const int *openhpsdr_ep2_c2_fields[] = {
	&hf_openhpsdr_mode,
	&hf_openhpsdr_oc0,
	&hf_openhpsdr_oc1,
	&hf_openhpsdr_oc2,
	&hf_openhpsdr_oc3,
	&hf_openhpsdr_oc4,
	&hf_openhpsdr_oc5,
	&hf_openhpsdr_oc6,
	NULL
};

/* C3 Fields from computer when C0 = 0x00 */
static int hf_openhpsdr_ep2_c3 = -1;
static int hf_openhpsdr_alex_atten = -1;
static int hf_openhpsdr_preamp = -1;
static int hf_openhpsdr_dither = -1;
static int hf_openhpsdr_random = -1;
static int hf_openhpsdr_alex_rx_antenna = -1;
static int hf_openhpsdr_alex_rx_out = -1;

static const int *openhpsdr_ep2_c3_fields[] = {
	&hf_openhpsdr_alex_atten,
	&hf_openhpsdr_preamp,
	&hf_openhpsdr_dither,
	&hf_openhpsdr_random,
	&hf_openhpsdr_alex_rx_antenna,
	&hf_openhpsdr_alex_rx_out,
	NULL
};

static const value_string attensettings[] = {
	{ 0, "0dB" },
	{ 1, "10dB" },
	{ 2, "20dB" },
	{ 3, "30dB" },
	{ 0, NULL }
};

static const value_string alexrxantenna[] = {
	{ 0, "none" },
	{ 1, "RX1" },
	{ 2, "RX2" },
	{ 3, "Transverter" },
	{ 0, NULL }
};

/* C4 fields from computer when C0 = 0x00 */
static int hf_openhpsdr_ep2_c4 = -1;
static int hf_openhpsdr_alex_tx_relay = -1;
static int hf_openhpsdr_duplex = -1;
static int hf_openhpsdr_receivers = -1;
static int hf_openhpsdr_1pps_timestamp = -1;
static int hf_openhpsdr_commonfreq = -1;

static const int *openhpsdr_ep2_c4_fields[] = {
	&hf_openhpsdr_alex_tx_relay,
	&hf_openhpsdr_duplex,
	&hf_openhpsdr_receivers,
	&hf_openhpsdr_1pps_timestamp,
	&hf_openhpsdr_commonfreq,
	NULL
};

static const value_string txrelay[] = {
	{ 0, "Tx1" },
	{ 1, "Tx2" },
	{ 3, "Tx3" },
	{ 4, "Undefined" },
	{ 0, NULL }
};

static const true_false_string tfs_ind_dep = {
	"Independent Frequencies",
	"Same Frequency"
}; 

/* ------ */

static const value_string commandnames[] = {
	{ 1, "Data" },
	{ 2, "Discover" },
	{ 3, "Busy" },
	{ 4, "Start/Stop" },
	{ 0, NULL }
};

static const value_string boardids[] = {
	{ 0, "Metis" },
	{ 1, "Hermes" },
	{ 2, "Griffin" },
	{ 3, "Angelia" },
	{ 0, NULL }
};

static const value_string speeds[] = {
	{ 0, "48kHz" },
	{ 1, "96kHz" },
	{ 2, "192kHz" },
	{ 3, "384kHz" },
	{ 0, NULL }
};

static const value_string references10[] = {
	{ 0, "Atlas/Excalibur" },
	{ 1, "Penelope" },
	{ 2, "Mercury" },
	{ 0, NULL }
};

static const value_string references122[] = {
	{ 0, "Penelope" },
	{ 1, "Mercury" },
	{ 0, NULL }
};

static const value_string configs[] = {
	{ 0, "nil" },
	{ 1, "Penelope" },
	{ 2, "Mercury" },
	{ 3, "Penelope & Mercury" },
	{ 0, NULL }
};

static const value_string mic_sources[] = {
	{ 0, "Janus" },
	{ 1, "Penelope" },
	{ 0, NULL }
};

static const int *openhpsdr_ep6_c0_fields[] = {
	&hf_openhpsdr_ptt,
	&hf_openhpsdr_dash,
	&hf_openhpsdr_dot,
	&hf_openhpsdr_usbpacket_ep6_rr,
	NULL
};

static const int *openhpsdr_ep2_c0_fields[] = {
	&hf_openhpsdr_mox,
	&hf_openhpsdr_usbpacket_ep2_rr,
	NULL
};

static const true_false_string tfs_inactive_active = {
	"Inactive",
	"Active"
};

static const true_false_string tfs_openhpsdr_active_inactive = {
	"Active",
	"Inactive"
};

static const true_false_string tfs_locked_unlocked = {
	"Locked",
	"Unlocked"
};

static const true_false_string tfs_classe_other = {
	"Class E",
	"Other"
};

static const true_false_string tfs_openhpsdr_on_off = {
	"On",
	"Off"
};

static void
dissect_openhpsdr_hardware_usbpacket(tvbuff_t *tvb, proto_tree *tree, gint offset) {
	proto_item *local_item;
	proto_tree *local_tree;
	guint8 c0;
	guint8 version;

	local_item = proto_tree_add_text(tree, tvb, offset, 512, "USB Packet");
	local_tree = proto_item_add_subtree(local_item, ett_openhpsdr_usbpacket);
	offset += 3;

	proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_usbpacket_c0, ett_openhpsdr_c0, openhpsdr_ep6_c0_fields, ENC_BIG_ENDIAN);

	c0 = tvb_get_guint8(tvb, offset) >> 3;
	++offset;
	switch(c0) {
		case 0x00:
			proto_tree_add_item(local_tree, hf_openhpsdr_overflow, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_i01, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_i02, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_i03, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_i04, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_cyclops_lock, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_item(local_tree, hf_openhpsdr_cyclops_frequency_changed, tvb, offset, 1, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_item(local_tree, hf_openhpsdr_usbpacket_mercury_version, tvb, offset, 1, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_item(local_tree, hf_openhpsdr_usbpacket_penelope_version, tvb, offset, 1, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_item(local_tree, hf_openhpsdr_usbpacket_metis_version, tvb, offset, 1, ENC_BIG_ENDIAN);
			break;
		case 0x01:
			proto_tree_add_item(local_tree, hf_openhpsdr_hermes_forward_power, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			proto_tree_add_item(local_tree, hf_openhpsdr_apollo_forward_power, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			break;
		case 0x02:
			proto_tree_add_item(local_tree, hf_openhpsdr_apollo_reverse_power, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			proto_tree_add_item(local_tree, hf_openhpsdr_ain3, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			break;
		case 0x03:
			proto_tree_add_item(local_tree, hf_openhpsdr_ain4, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			proto_tree_add_item(local_tree, hf_openhpsdr_ain6, tvb, offset, 2, ENC_BIG_ENDIAN);
			offset += 2;
			break;
		case 0x04:
			version = tvb_get_guint8(tvb, offset);
			proto_tree_add_item(local_tree, hf_openhpsdr_merc1_overflow, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_boolean_format_value(local_tree, hf_openhpsdr_merc1_version, tvb, offset, 1, version, "version = %d", version >> 1);
			++offset;
			version = tvb_get_guint8(tvb, offset);
			proto_tree_add_item(local_tree, hf_openhpsdr_merc2_overflow, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_boolean_format_value(local_tree, hf_openhpsdr_merc2_version, tvb, offset, 1, version, "version = %d", version >> 1);
			++offset;
			version = tvb_get_guint8(tvb, offset);
			proto_tree_add_item(local_tree, hf_openhpsdr_merc3_overflow, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_boolean_format_value(local_tree, hf_openhpsdr_merc3_version, tvb, offset, 1, version, "version = %d", version >> 1);
			++offset;
			version = tvb_get_guint8(tvb, offset);
			proto_tree_add_item(local_tree, hf_openhpsdr_merc4_overflow, tvb, offset, 1, ENC_BIG_ENDIAN);
			proto_tree_add_boolean_format_value(local_tree, hf_openhpsdr_merc4_version, tvb, offset, 1, version, "version = %d", version >> 1);
			++offset;
	}

	proto_tree_add_item(local_tree, hf_openhpsdr_sample_data, tvb, offset, 504, ENC_BIG_ENDIAN);
}

static void
dissect_openhpsdr_computer_usbpacket(tvbuff_t *tvb, proto_tree *tree, gint offset) {
	proto_item *local_item;
	proto_tree *local_tree;
	guint8 c0;

	local_item = proto_tree_add_text(tree, tvb, offset, 512, "USB Packet");
	local_tree = proto_item_add_subtree(local_item, ett_openhpsdr_usbpacket);
	offset += 3;

	proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_usbpacket_c0, ett_openhpsdr_c0, openhpsdr_ep2_c0_fields, ENC_BIG_ENDIAN);

	c0 = tvb_get_guint8(tvb, offset) >> 1;
	++offset;
	switch(c0) {
		/* XXX Parse EP2 packets here */
		case 0x00:
			proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_ep2_c1, ett_openhpsdr_c1, openhpsdr_ep2_c1_fields, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_ep2_c2, ett_openhpsdr_c2, openhpsdr_ep2_c2_fields, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_ep2_c3, ett_openhpsdr_c3, openhpsdr_ep2_c3_fields, ENC_BIG_ENDIAN);
			++offset;
			proto_tree_add_bitmask(local_tree, tvb, offset, hf_openhpsdr_ep2_c4, ett_openhpsdr_c4, openhpsdr_ep2_c4_fields, ENC_BIG_ENDIAN);
			++offset;
		break;
	}
	proto_tree_add_item(local_tree, hf_openhpsdr_sample_data, tvb, offset, 504, ENC_BIG_ENDIAN);

}

static void
dissect_openhpsdr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
	gint offset = 2;
	guint8 command;
	guint8 endpoint = 0;
	guint16 magic;
	guint64 ether_mac = 0;

	proto_item *local_item;
	proto_tree *local_tree;

	magic = tvb_get_ntohs(tvb, 0);
	if(magic != 0xeffe) return;

	col_set_str(pinfo->cinfo, COL_PROTOCOL, "OpenHPSDR");
	col_clear(pinfo->cinfo, COL_INFO);

	command = tvb_get_guint8(tvb, offset);
	switch(command) {
		case 0x01:
			endpoint = tvb_get_guint8(tvb, offset+1);
			col_append_fstr(pinfo->cinfo, COL_INFO, "Data (0x01) Endpoint = %u", endpoint);
			break;
		case 0x02:
			col_append_str(pinfo->cinfo, COL_INFO, "Discovery (0x02)");
			break;
		case 0x03:
			col_append_str(pinfo->cinfo, COL_INFO, "Busy (0x03)");
			break;
		case 0x04:
			col_append_str(pinfo->cinfo, COL_INFO, "Start/Stop (0x04)");
			break;
		default:
			col_append_str(pinfo->cinfo, COL_INFO, "Unknown Command");
	}

	if(tree) {
		proto_item *ti = NULL;
		proto_tree *openhpsdr_tree = NULL;

		ti = proto_tree_add_item(tree, proto_openhpsdr, tvb, 0, -1, ENC_NA);
		openhpsdr_tree = proto_item_add_subtree(ti, ett_openhpsdr);
		proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_command, tvb, offset, 1, ENC_BIG_ENDIAN);
		++offset;
		
		/*  Metis MAC Address if Discovery Reply */
		switch(command) {
			case 0x01:
				proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_endpoint, tvb, offset, 1, ENC_BIG_ENDIAN);
				++offset;
				proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_sequence, tvb, offset, 4, ENC_BIG_ENDIAN);
				offset += 4;
				if(endpoint == 0x04 || endpoint == 0x06) {
					dissect_openhpsdr_hardware_usbpacket(tvb, openhpsdr_tree, offset);
					offset += 512;
					dissect_openhpsdr_hardware_usbpacket(tvb, openhpsdr_tree, offset);
				} else if(endpoint == 0x02) {
					dissect_openhpsdr_computer_usbpacket(tvb, openhpsdr_tree, offset);
					offset += 512;
					dissect_openhpsdr_computer_usbpacket(tvb, openhpsdr_tree, offset);
				}
				break;
			case 0x02:
			case 0x03:
				ether_mac = tvb_get_ntoh48(tvb, offset);
				if(ether_mac != 0) {
					proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_metis_mac, tvb, offset, 6, ENC_BIG_ENDIAN);
					offset += 6;
					proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_metis_version, tvb, offset, 1, ENC_BIG_ENDIAN);
					++offset;
					proto_tree_add_item(openhpsdr_tree, hf_openhpsdr_board_id, tvb, offset, 1, ENC_BIG_ENDIAN);
				}
				break;
			case 0x04:
				local_item = proto_tree_add_text(openhpsdr_tree, tvb, offset, 1, "Flags");
				local_tree = proto_item_add_subtree(local_item, ett_openhpsdr_flags);
				proto_tree_add_item(local_tree, hf_openhpsdr_ep4_state, tvb, offset, 1, ENC_BIG_ENDIAN);
				proto_tree_add_item(local_tree, hf_openhpsdr_ep6_state, tvb, offset, 1, ENC_BIG_ENDIAN);
				++offset;
				break;
		}
		
	}
}

void
proto_register_openhpsdr(void)
{
	static hf_register_info hf[] = {
		{ &hf_openhpsdr_command,
			{ "Command", "openhpsdr.command",
			FT_UINT8, BASE_HEX,
			VALS(commandnames), 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_metis_mac,
			{ "Metis MAC", "openhpsdr.mac",
			FT_ETHER, BASE_NONE,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_metis_version,
			{ "Metis Version", "openhpsdr.version",
			FT_UINT8, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_board_id,
			{ "Board ID", "openhpsdr.boardid",
			FT_UINT8, BASE_HEX,
			VALS(boardids), 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_ep4_state,
			{ "EP4 State", "openhpsdr.ep4",
			FT_BOOLEAN, 8, 
			NULL, 0x01,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_ep6_state,
			{ "EP6 State", "openhpsdr.ep6",
			FT_BOOLEAN, 8,
			NULL, 0x02,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_ptt,
			{ "PTT", "openhpsdr.usbpacket.ptt",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_active_inactive), 0x01,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_dash,
			{ "Dash", "openhpsdr.usbpacket.dash",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_active_inactive), 0x02,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_dot,
			{ "Dot", "openhpsdr.usbpacket.dot",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_active_inactive), 0x04,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_endpoint,
			{ "Endpoint", "openhpsdr.endpoint",
			FT_UINT8, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_sequence,
			{ "Sequence Number", "openhpsdr.sequence",
			FT_UINT32, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_usbpacket_c0,
			{ "C0", "openhpsdr.usbpacket.c0",
			FT_UINT8, BASE_HEX,
			NULL, 0x0,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_usbpacket_ep6_rr,
			{ "Round Robin", "openhpsdr.usbpacket.rr",
			FT_UINT8, BASE_HEX,
			NULL, 0xF8,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_usbpacket_ep2_rr,
			{ "Round Robin", "openhpsdr.usbpacket.rr",
			FT_UINT8, BASE_HEX,
			NULL, 0xFE,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_mox,
			{ "MOX", "openhpsdr.usbpacket.mox",
			FT_BOOLEAN, 8,
			NULL, 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_overflow,
			{ "LTC2208 Overflow", "openhpsdr.usbpacket.overflow",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_active_inactive), 0x01,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_i01,
			{ "Hermes I01", "openhpsdr.usbpacket.i01",
			FT_BOOLEAN, 8,
			TFS(&tfs_inactive_active), 0x02,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_i02,
			{ "Hermes I02", "openhpsdr.usbpacket.i02",
			FT_BOOLEAN, 8,
			TFS(&tfs_inactive_active), 0x04,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_i03,
			{ "Hermes I03", "openhpsdr.usbpacket.i03",
			FT_BOOLEAN, 8,
			TFS(&tfs_inactive_active), 0x08,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_i04,
			{ "Hermes I04", "openhpsdr.usbpacket.i04",
			FT_BOOLEAN, 8,
			TFS(&tfs_inactive_active), 0x10,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_cyclops_lock,
			{ "Cyclops PLL Locked", "openhpsdr.usbpacket.cyclopslock",
			FT_BOOLEAN, 8,
			TFS(&tfs_locked_unlocked), 0x20,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_cyclops_frequency_changed,
			{ "Cyclops Frequency Changed", "openhpsdr.usbpacket.cyclopsfreqchanged",
			FT_BOOLEAN, 8,
			NULL, 0x40,
			NULL, HFILL }
		}, 
		{ &hf_openhpsdr_usbpacket_metis_version,
			{ "Metis Version", "openhpsdr.usbpacket.metisversion",
			FT_UINT8, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_usbpacket_penelope_version,
			{ "Penelope Version", "openhpsdr.usbpacket.pennyversion",
			FT_UINT8, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_usbpacket_mercury_version,
			{ "Mercury Version", "openhpsdr.usbpacket.mercuryversion",
			FT_UINT8, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_hermes_forward_power,
			{ "Hermes Forward Power", "openhpsdr.usbpacket.hermesforwardpower",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_apollo_forward_power,
			{ "Apollo Forward Power", "openhpsdr.usbpacket.apolloforwardpower",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_apollo_reverse_power,
			{ "Apollo Reverse Power", "openhpsdr.usbpacket.apolloreversepower",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_ain3,
			{ "AIN3", "openhpsdr.usbpacket.ain3",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_ain4,
			{ "AIN4", "openhpsdr.usbpacket.ain4",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_ain6,
			{ "AIN6", "openhpsdr.usbpacket.ain6",
			FT_UINT16, BASE_DEC,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_openhpsdr_merc1_version,
			{ "Mercury 1 Version", "openhpsdr.usbpacket.merc1.version",
			FT_BOOLEAN, 8,
			NULL, 0xFE,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc1_overflow,
			{ "Mercury 1 Overflow", "openhpsdr.usbpacket.merc1.overflow",
			FT_BOOLEAN, 8,
			NULL, 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc2_version,
			{ "Mercury 2 Version", "openhpsdr.usbpacket.merc2.version",
			FT_BOOLEAN, 8,
			NULL, 0xFE,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc2_overflow,
			{ "Mercury 2 Overflow", "openhpsdr.usbpacket.merc2.overflow",
			FT_BOOLEAN, 8,
			NULL, 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc3_version,
			{ "Mercury 3 Version", "openhpsdr.usbpacket.merc3.version",
			FT_BOOLEAN, 8,
			NULL, 0xFE,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc3_overflow,
			{ "Mercury 3 Overflow", "openhpsdr.usbpacket.merc3.overflow",
			FT_BOOLEAN, 8,
			NULL, 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc4_version,
			{ "Mercury 4 Version", "openhpsdr.usbpacket.merc4.version",
			FT_BOOLEAN, 8,
			NULL, 0xFE,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_merc4_overflow,
			{ "Mercury 4 Overflow", "openhpsdr.usbpacket.merc4.overflow",
			FT_BOOLEAN, 8,
			NULL, 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_ep2_c1,
			{ "C1", "openhpsdr.usbpacket.c1",
			FT_UINT8, BASE_HEX,
			NULL, 0x00,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_speed,
			{ "Speed", "openhpsdr.usbpacket.speed",
			FT_UINT8, BASE_HEX,
			VALS(&speeds), 0x03,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_10mhz,
			{ "10MHz Reference", "openhpsdr.usbpacket.10mhz",
			FT_UINT8, BASE_HEX,
			VALS(&references10), 0x0C,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_122mhz,
			{ "122.88MHz Reference", "openhpsdr.usbpacket.122mhz",
			FT_UINT8, BASE_HEX,
			VALS(&references122), 0x10,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_config,
			{ "Config", "openhpsdr.usbpacket.config",
			FT_UINT8, BASE_HEX,
			VALS(&configs), 0x60,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_mic_source,
			{ "Mic Source", "openhpsdr.usbpacket.micsource",
			FT_UINT8, BASE_HEX,
			VALS(&mic_sources), 0x80,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_ep2_c2,
			{ "C2", "openhpsdr.usbpacket.c2",
			FT_UINT8, BASE_HEX,
			NULL, 0x00,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_mode,
			{ "Mode", "openhpsdr.usbpacket.mode",
			FT_BOOLEAN, 8,
			TFS(&tfs_classe_other), 0x01,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc0,
			{ "Open Collector 0", "openhpsdr.usbpacket.oc0",
			FT_BOOLEAN, 8,
			NULL, 0x02,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc1,
			{ "Open Collector 1", "openhpsdr.usbpacket.oc1",
			FT_BOOLEAN, 8,
			NULL, 0x04,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc2,
			{ "Open Collector 2", "openhpsdr.usbpacket.oc2",
			FT_BOOLEAN, 8,
			NULL, 0x08,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc3,
			{ "Open Collector 3", "openhpsdr.usbpacket.oc3",
			FT_BOOLEAN, 8,
			NULL, 0x10,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc4,
			{ "Open Collector 4", "openhpsdr.usbpacket.oc4",
			FT_BOOLEAN, 8,
			NULL, 0x20,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc5,
			{ "Open Collector 5", "openhpsdr.usbpacket.oc5",
			FT_BOOLEAN, 8,
			NULL, 0x40,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_oc6,
			{ "Open Collector 6", "openhpsdr.usbpacket.oc6",
			FT_BOOLEAN, 8,
			NULL, 0x80,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_ep2_c3,
			{ "C3", "openhpsdr.usbpacket.c3",
			FT_UINT8, BASE_HEX,
			NULL, 0x00,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_alex_atten,
			{ "Alex Attenuator", "openhpsdr.usbpacket.alexatten",
			FT_UINT8, BASE_HEX,
			VALS(&attensettings), 0x03,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_preamp,
			{ "Preamp", "openhpsdr.usbpacket.preamp",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x04,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_dither,
			{ "LTC2208 Dither", "openhpsdr.usbpacket.dither",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x08,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_random,
			{ "LTC2208 Random", "openhpsdr.usbpacket.random",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x10,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_alex_rx_antenna,
			{ "Alex Rx Antenna", "openhpsdr.usbpacket.alexrxantenna",
			FT_UINT8, BASE_HEX,
			VALS(&alexrxantenna), 0x60,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_alex_rx_out,
			{ "Alex Rx Out", "openhpsdr.usbpacket.alexrxout",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x80,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_ep2_c4,
			{ "C4", "openhpsdr.usbpacket.c4",
			FT_UINT8, BASE_HEX,
			NULL, 0x00,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_alex_tx_relay,
			{ "Alex Tx Relay", "openhpsdr.usbpacket.alextxrelay",
			FT_UINT8, BASE_HEX,
			VALS(&txrelay), 0x03,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_duplex,
			{ "Duplex", "openhpsdr.usbpacket.duplex",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x04,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_receivers,
			{ "Number of Receivers", "openhpsdr.usbpacket.receivers",
			FT_UINT8, BASE_HEX,
			NULL, 0x38,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_1pps_timestamp,
			{ "1PPS Timestamp", "openhpsdr.usbpacket.1ppstimestamp",
			FT_BOOLEAN, 8,
			TFS(&tfs_openhpsdr_on_off), 0x40,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_commonfreq,
			{ "Common Mercury Frequency", "openhpsdr.usbpacket.commonfreq",
			FT_BOOLEAN, 8,
			TFS(&tfs_ind_dep), 0x80,
			NULL, HFILL }
		},	
		{ &hf_openhpsdr_sample_data,
			{ "Sample Data", "openhpsdr.usbpacket.sampledata",
			FT_BYTES, BASE_NONE,
			NULL, 0x0,
			NULL, HFILL }
		},	
	};

	static gint *ett[] = {
		&ett_openhpsdr,
		&ett_openhpsdr_flags,
		&ett_openhpsdr_usbpacket,
		&ett_openhpsdr_c0tree,
		&ett_openhpsdr_c0,
		&ett_openhpsdr_c1,
		&ett_openhpsdr_c2,
		&ett_openhpsdr_c3,
		&ett_openhpsdr_c4
	};

	proto_openhpsdr = proto_register_protocol (
		"OpenHPSDR Ethernet Protocol",
		"OpenHPSDR",
		"openhpsdr"
	);

	proto_register_field_array(proto_openhpsdr, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
}

void
proto_reg_handoff_openhpsdr(void)
{
	static dissector_handle_t openhpsdr_handle;

	openhpsdr_handle = create_dissector_handle(dissect_openhpsdr, proto_openhpsdr);
	dissector_add_uint("udp.port", OPENHPSDR_PORT, openhpsdr_handle);
}
